# -*- coding: utf-8 -*-

from distutils.core import setup

setup(
    name='iterpipes',
    version='0.1',
    py_modules=['iterpipes'],
    author='Andrey Vlasovskikh',
    author_email='andrey.vlasovskikh@gmail.com',
    description='A library for running shell pipelines using shell-like syntax',
    license='MIT',
    url='http://bitbucket.org/vlasovskikh/iterpipes/')

